# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_supplemental_signals_response_content import GetSupplementalSignalsResponseContent
from ..types.patch_supplemental_signals_response_content import PatchSupplementalSignalsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSupplementalSignalsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSupplementalSignalsResponseContent]:
        """
        Get the supplemental signals configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSupplementalSignalsResponseContent]
            Supplemental Signals configuration successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "supplemental-signals",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSupplementalSignalsResponseContent,
                    parse_obj_as(
                        type_=GetSupplementalSignalsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def patch(
        self, *, akamai_enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PatchSupplementalSignalsResponseContent]:
        """
        Update the supplemental signals configuration for a tenant.

        Parameters
        ----------
        akamai_enabled : bool
            Indicates if incoming Akamai Headers should be processed

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatchSupplementalSignalsResponseContent]
            Supplemental Signals configuration successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "supplemental-signals",
            method="PATCH",
            json={
                "akamai_enabled": akamai_enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PatchSupplementalSignalsResponseContent,
                    parse_obj_as(
                        type_=PatchSupplementalSignalsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSupplementalSignalsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSupplementalSignalsResponseContent]:
        """
        Get the supplemental signals configuration for a tenant.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSupplementalSignalsResponseContent]
            Supplemental Signals configuration successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "supplemental-signals",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSupplementalSignalsResponseContent,
                    parse_obj_as(
                        type_=GetSupplementalSignalsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def patch(
        self, *, akamai_enabled: bool, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PatchSupplementalSignalsResponseContent]:
        """
        Update the supplemental signals configuration for a tenant.

        Parameters
        ----------
        akamai_enabled : bool
            Indicates if incoming Akamai Headers should be processed

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatchSupplementalSignalsResponseContent]
            Supplemental Signals configuration successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "supplemental-signals",
            method="PATCH",
            json={
                "akamai_enabled": akamai_enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PatchSupplementalSignalsResponseContent,
                    parse_obj_as(
                        type_=PatchSupplementalSignalsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .breached_password_detection_pre_change_password_stage import BreachedPasswordDetectionPreChangePasswordStage
from .breached_password_detection_pre_user_registration_stage import BreachedPasswordDetectionPreUserRegistrationStage


class BreachedPasswordDetectionStage(UniversalBaseModel):
    pre_user_registration: typing_extensions.Annotated[
        typing.Optional[BreachedPasswordDetectionPreUserRegistrationStage],
        FieldMetadata(alias="pre-user-registration"),
        pydantic.Field(alias="pre-user-registration"),
    ] = None
    pre_change_password: typing_extensions.Annotated[
        typing.Optional[BreachedPasswordDetectionPreChangePasswordStage],
        FieldMetadata(alias="pre-change-password"),
        pydantic.Field(alias="pre-change-password"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mdl_presentation_request import MdlPresentationRequest


class CreateVerifiableCredentialTemplateResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The id of the template.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the template.
    """

    type: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of the template.
    """

    dialect: typing.Optional[str] = pydantic.Field(default=None)
    """
    The dialect of the template.
    """

    presentation: typing.Optional[MdlPresentationRequest] = None
    custom_certificate_authority: typing.Optional[str] = pydantic.Field(default=None)
    """
    The custom certificate authority.
    """

    well_known_trusted_issuers: typing.Optional[str] = pydantic.Field(default=None)
    """
    The well-known trusted issuers, comma separated.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time the template was created.
    """

    updated_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The date and time the template was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .rate_limit_policy_configuration_action import RateLimitPolicyConfigurationAction
from .rate_limit_policy_configuration_one import RateLimitPolicyConfigurationOne
from .rate_limit_policy_configuration_zero import RateLimitPolicyConfigurationZero

RateLimitPolicyConfiguration = typing.Union[
    RateLimitPolicyConfigurationZero, RateLimitPolicyConfigurationOne, RateLimitPolicyConfigurationAction
]



# This file was auto-generated by Fern from our API Definition.

ConnectionForwardReqInfoSms = bool
"""
When set to true, the gateway receives HTTP request details including IP address and User Agent from the client.
"""



from .conftest import get_client, verify_request_count


def test_emailTemplates_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "email_templates.create.0"
    client = get_client(test_id)
    client.email_templates.create(
        template="verify_email",
    )
    verify_request_count(test_id, "POST", "/email-templates", None, 1)


def test_emailTemplates_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "email_templates.get.0"
    client = get_client(test_id)
    client.email_templates.get(
        template_name="verify_email",
    )
    verify_request_count(test_id, "GET", "/email-templates/verify_email", None, 1)


def test_emailTemplates_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "email_templates.set_.0"
    client = get_client(test_id)
    client.email_templates.set(
        template_name="verify_email",
        template="verify_email",
    )
    verify_request_count(test_id, "PUT", "/email-templates/verify_email", None, 1)


def test_emailTemplates_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "email_templates.update.0"
    client = get_client(test_id)
    client.email_templates.update(
        template_name="verify_email",
    )
    verify_request_count(test_id, "PATCH", "/email-templates/verify_email", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

LogStreamEventBridgeSinkRegionEnum = typing.Union[
    typing.Literal[
        "af-south-1",
        "ap-east-1",
        "ap-east-2",
        "ap-northeast-1",
        "ap-northeast-2",
        "ap-northeast-3",
        "ap-south-1",
        "ap-south-2",
        "ap-southeast-1",
        "ap-southeast-2",
        "ap-southeast-3",
        "ap-southeast-4",
        "ap-southeast-5",
        "ap-southeast-6",
        "ap-southeast-7",
        "ca-central-1",
        "ca-west-1",
        "eu-central-1",
        "eu-central-2",
        "eu-north-1",
        "eu-south-1",
        "eu-south-2",
        "eu-west-1",
        "eu-west-2",
        "eu-west-3",
        "il-central-1",
        "me-central-1",
        "me-south-1",
        "mx-central-1",
        "sa-east-1",
        "us-gov-east-1",
        "us-gov-west-1",
        "us-east-1",
        "us-east-2",
        "us-west-1",
        "us-west-2",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ConnectionGatewayAuthentication(UniversalBaseModel):
    """
    Token-based authentication settings to be applied when connection is using an sms strategy.
    """

    method: str = pydantic.Field()
    """
    The Authorization header type.
    """

    subject: typing.Optional[str] = pydantic.Field(default=None)
    """
    The subject to be added to the JWT payload.
    """

    audience: str = pydantic.Field()
    """
    The audience to be added to the JWT payload.
    """

    secret: str = pydantic.Field()
    """
    The secret to be used for signing tokens.
    """

    secret_base_64_encoded: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="secret_base64_encoded"),
        pydantic.Field(
            alias="secret_base64_encoded", description="Set to true if the provided secret is base64 encoded."
        ),
    ] = None
    """
    Set to true if the provided secret is base64 encoded.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.connection_for_organization import ConnectionForOrganization
from ..types.create_organization_response_content import CreateOrganizationResponseContent
from ..types.create_token_quota import CreateTokenQuota
from ..types.get_organization_by_name_response_content import GetOrganizationByNameResponseContent
from ..types.get_organization_response_content import GetOrganizationResponseContent
from ..types.list_organizations_paginated_response_content import ListOrganizationsPaginatedResponseContent
from ..types.organization import Organization
from ..types.organization_branding import OrganizationBranding
from ..types.organization_metadata import OrganizationMetadata
from ..types.organization_third_party_client_access_enum import OrganizationThirdPartyClientAccessEnum
from ..types.update_organization_response_content import UpdateOrganizationResponseContent
from ..types.update_token_quota import UpdateTokenQuota
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawOrganizationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Organization, ListOrganizationsPaginatedResponseContent]:
        """
        Retrieve detailed list of all Organizations available in your tenant. For more information, see Auth0 Organizations.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organizations.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total number of entries to retrieve when using the `from` parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>. We currently support sorting by the following fields: <code>name</code>, <code>display_name</code> and <code>created_at</code>.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Organization, ListOrganizationsPaginatedResponseContent]
            Organizations successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "organizations",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrganizationsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListOrganizationsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.organizations
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""
                _get_next = lambda: self.list(
                    from_=_parsed_next,
                    take=take,
                    sort=sort,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        display_name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        enabled_connections: typing.Optional[typing.Sequence[ConnectionForOrganization]] = OMIT,
        token_quota: typing.Optional[CreateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateOrganizationResponseContent]:
        """
        Create a new Organization within your tenant.  To learn more about Organization settings, behavior, and configuration options, review [Create Your First Organization](https://auth0.com/docs/manage-users/organizations/create-first-organization).

        Parameters
        ----------
        name : str
            The name of this organization.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        enabled_connections : typing.Optional[typing.Sequence[ConnectionForOrganization]]
            Connections that will be enabled for this organization. See POST enabled_connections endpoint for the object format. (Max of 10 connections allowed)

        token_quota : typing.Optional[CreateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateOrganizationResponseContent]
            Organization successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "organizations",
            method="POST",
            json={
                "name": name,
                "display_name": display_name,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=OrganizationBranding, direction="write"
                ),
                "metadata": metadata,
                "enabled_connections": convert_and_respect_annotation_metadata(
                    object_=enabled_connections,
                    annotation=typing.Sequence[ConnectionForOrganization],
                    direction="write",
                ),
                "token_quota": convert_and_respect_annotation_metadata(
                    object_=token_quota, annotation=CreateTokenQuota, direction="write"
                ),
                "third_party_client_access": third_party_client_access,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrganizationResponseContent,
                    parse_obj_as(
                        type_=CreateOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_by_name(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetOrganizationByNameResponseContent]:
        """
        Retrieve details about a single Organization specified by name.

        Parameters
        ----------
        name : str
            name of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetOrganizationByNameResponseContent]
            Organization successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/name/{encode_path_param(name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetOrganizationByNameResponseContent,
                    parse_obj_as(
                        type_=GetOrganizationByNameResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetOrganizationResponseContent]:
        """
        Retrieve details about a single Organization specified by ID.

        Parameters
        ----------
        id : str
            ID of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetOrganizationResponseContent]
            Organization successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetOrganizationResponseContent,
                    parse_obj_as(
                        type_=GetOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Remove an Organization from your tenant.  This action cannot be undone.

        **Note**: Members are automatically disassociated from an Organization when it is deleted. However, this action does **not** delete these users from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        token_quota: typing.Optional[UpdateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateOrganizationResponseContent]:
        """
        Update the details of a specific [Organization](https://auth0.com/docs/manage-users/organizations/configure-organizations/create-organizations), such as name and display name, branding options, and metadata.

        Parameters
        ----------
        id : str
            ID of the organization to update.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        name : typing.Optional[str]
            The name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        token_quota : typing.Optional[UpdateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateOrganizationResponseContent]
            Organization successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="PATCH",
            json={
                "display_name": display_name,
                "name": name,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=OrganizationBranding, direction="write"
                ),
                "metadata": metadata,
                "token_quota": convert_and_respect_annotation_metadata(
                    object_=token_quota, annotation=typing.Optional[UpdateTokenQuota], direction="write"
                ),
                "third_party_client_access": third_party_client_access,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrganizationResponseContent,
                    parse_obj_as(
                        type_=UpdateOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawOrganizationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Organization, ListOrganizationsPaginatedResponseContent]:
        """
        Retrieve detailed list of all Organizations available in your tenant. For more information, see Auth0 Organizations.

        This endpoint supports two types of pagination:

        - Offset pagination
        - Checkpoint pagination

        Checkpoint pagination must be used if you need to retrieve more than 1000 organizations.

        **Checkpoint Pagination**

        To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total number of entries to retrieve when using the `from` parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>. We currently support sorting by the following fields: <code>name</code>, <code>display_name</code> and <code>created_at</code>.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Organization, ListOrganizationsPaginatedResponseContent]
            Organizations successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "organizations",
            method="GET",
            params={
                "from": from_,
                "take": take,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListOrganizationsPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListOrganizationsPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.organizations
                _parsed_next = _parsed_response.next
                _has_next = _parsed_next is not None and _parsed_next != ""

                async def _get_next():
                    return await self.list(
                        from_=_parsed_next,
                        take=take,
                        sort=sort,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        display_name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        enabled_connections: typing.Optional[typing.Sequence[ConnectionForOrganization]] = OMIT,
        token_quota: typing.Optional[CreateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateOrganizationResponseContent]:
        """
        Create a new Organization within your tenant.  To learn more about Organization settings, behavior, and configuration options, review [Create Your First Organization](https://auth0.com/docs/manage-users/organizations/create-first-organization).

        Parameters
        ----------
        name : str
            The name of this organization.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        enabled_connections : typing.Optional[typing.Sequence[ConnectionForOrganization]]
            Connections that will be enabled for this organization. See POST enabled_connections endpoint for the object format. (Max of 10 connections allowed)

        token_quota : typing.Optional[CreateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateOrganizationResponseContent]
            Organization successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "organizations",
            method="POST",
            json={
                "name": name,
                "display_name": display_name,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=OrganizationBranding, direction="write"
                ),
                "metadata": metadata,
                "enabled_connections": convert_and_respect_annotation_metadata(
                    object_=enabled_connections,
                    annotation=typing.Sequence[ConnectionForOrganization],
                    direction="write",
                ),
                "token_quota": convert_and_respect_annotation_metadata(
                    object_=token_quota, annotation=CreateTokenQuota, direction="write"
                ),
                "third_party_client_access": third_party_client_access,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateOrganizationResponseContent,
                    parse_obj_as(
                        type_=CreateOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_by_name(
        self, name: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetOrganizationByNameResponseContent]:
        """
        Retrieve details about a single Organization specified by name.

        Parameters
        ----------
        name : str
            name of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetOrganizationByNameResponseContent]
            Organization successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/name/{encode_path_param(name)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetOrganizationByNameResponseContent,
                    parse_obj_as(
                        type_=GetOrganizationByNameResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetOrganizationResponseContent]:
        """
        Retrieve details about a single Organization specified by ID.

        Parameters
        ----------
        id : str
            ID of the organization to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetOrganizationResponseContent]
            Organization successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetOrganizationResponseContent,
                    parse_obj_as(
                        type_=GetOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Remove an Organization from your tenant.  This action cannot be undone.

        **Note**: Members are automatically disassociated from an Organization when it is deleted. However, this action does **not** delete these users from your tenant.

        Parameters
        ----------
        id : str
            Organization identifier.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        display_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        branding: typing.Optional[OrganizationBranding] = OMIT,
        metadata: typing.Optional[OrganizationMetadata] = OMIT,
        token_quota: typing.Optional[UpdateTokenQuota] = OMIT,
        third_party_client_access: typing.Optional[OrganizationThirdPartyClientAccessEnum] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateOrganizationResponseContent]:
        """
        Update the details of a specific [Organization](https://auth0.com/docs/manage-users/organizations/configure-organizations/create-organizations), such as name and display name, branding options, and metadata.

        Parameters
        ----------
        id : str
            ID of the organization to update.

        display_name : typing.Optional[str]
            Friendly name of this organization.

        name : typing.Optional[str]
            The name of this organization.

        branding : typing.Optional[OrganizationBranding]

        metadata : typing.Optional[OrganizationMetadata]

        token_quota : typing.Optional[UpdateTokenQuota]

        third_party_client_access : typing.Optional[OrganizationThirdPartyClientAccessEnum]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateOrganizationResponseContent]
            Organization successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"organizations/{encode_path_param(id)}",
            method="PATCH",
            json={
                "display_name": display_name,
                "name": name,
                "branding": convert_and_respect_annotation_metadata(
                    object_=branding, annotation=OrganizationBranding, direction="write"
                ),
                "metadata": metadata,
                "token_quota": convert_and_respect_annotation_metadata(
                    object_=token_quota, annotation=typing.Optional[UpdateTokenQuota], direction="write"
                ),
                "third_party_client_access": third_party_client_access,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateOrganizationResponseContent,
                    parse_obj_as(
                        type_=UpdateOrganizationResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .bot_detection_allowlist import BotDetectionAllowlist
from .bot_detection_challenge_policy_password_flow_enum import BotDetectionChallengePolicyPasswordFlowEnum
from .bot_detection_challenge_policy_password_reset_flow_enum import BotDetectionChallengePolicyPasswordResetFlowEnum
from .bot_detection_challenge_policy_passwordless_flow_enum import BotDetectionChallengePolicyPasswordlessFlowEnum
from .bot_detection_level_enum import BotDetectionLevelEnum
from .bot_detection_monitoring_mode_enabled import BotDetectionMonitoringModeEnabled


class GetBotDetectionSettingsResponseContent(UniversalBaseModel):
    bot_detection_level: BotDetectionLevelEnum
    challenge_password_policy: BotDetectionChallengePolicyPasswordFlowEnum
    challenge_passwordless_policy: BotDetectionChallengePolicyPasswordlessFlowEnum
    challenge_password_reset_policy: BotDetectionChallengePolicyPasswordResetFlowEnum
    allowlist: BotDetectionAllowlist
    monitoring_mode_enabled: BotDetectionMonitoringModeEnabled

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_roles_users_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "roles.users.list_.0"
    client = get_client(test_id)
    client.roles.users.list(
        id="id",
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/roles/id/users", {"from": "from", "take": "1"}, 1)


def test_roles_users_assign() -> None:
    """Test assign endpoint with WireMock"""
    test_id = "roles.users.assign.0"
    client = get_client(test_id)
    client.roles.users.assign(
        id="id",
        users=["users"],
    )
    verify_request_count(test_id, "POST", "/roles/id/users", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .branding_theme_borders_buttons_style_enum import BrandingThemeBordersButtonsStyleEnum
from .branding_theme_borders_inputs_style_enum import BrandingThemeBordersInputsStyleEnum


class BrandingThemeBorders(UniversalBaseModel):
    button_border_radius: float = pydantic.Field()
    """
    Button border radius
    """

    button_border_weight: float = pydantic.Field()
    """
    Button border weight
    """

    buttons_style: BrandingThemeBordersButtonsStyleEnum
    input_border_radius: float = pydantic.Field()
    """
    Input border radius
    """

    input_border_weight: float = pydantic.Field()
    """
    Input border weight
    """

    inputs_style: BrandingThemeBordersInputsStyleEnum
    show_widget_shadow: bool = pydantic.Field()
    """
    Show widget shadow
    """

    widget_border_weight: float = pydantic.Field()
    """
    Widget border weight
    """

    widget_corner_radius: float = pydantic.Field()
    """
    Widget corner radius
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionSendBackChannelNonce = bool
"""
When true and type is 'back_channel', includes a cryptographic nonce in authorization requests to prevent replay attacks. The identity provider must include this nonce in the ID token for validation.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

BotDetectionChallengePolicyPasswordFlowEnum = typing.Union[typing.Literal["never", "when_risky", "always"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ......core.pagination import AsyncPager, SyncPager
from ......core.request_options import RequestOptions
from ......types.group import Group
from ......types.list_organization_member_role_source_groups_response_content import (
    ListOrganizationMemberRoleSourceGroupsResponseContent,
)
from .raw_client import AsyncRawGroupsClient, RawGroupsClient


class GroupsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGroupsClient
        """
        return self._raw_client

    def list(
        self,
        id: str,
        user_id: str,
        *,
        role_id: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Group, ListOrganizationMemberRoleSourceGroupsResponseContent]:
        """
        Lists the groups which grant the org member a given role.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to list role source groups for.

        role_id : str
            The role ID to get group sources for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Group, ListOrganizationMemberRoleSourceGroupsResponseContent]
            Organization member role source groups successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.organizations.members.effective_roles.sources.groups.list(
            id="id",
            user_id="user_id",
            from_="from",
            take=1,
            role_id="role_id",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            id, user_id, role_id=role_id, from_=from_, take=take, request_options=request_options
        )


class AsyncGroupsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGroupsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGroupsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGroupsClient
        """
        return self._raw_client

    async def list(
        self,
        id: str,
        user_id: str,
        *,
        role_id: str,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Group, ListOrganizationMemberRoleSourceGroupsResponseContent]:
        """
        Lists the groups which grant the org member a given role.

        Parameters
        ----------
        id : str
            Organization identifier.

        user_id : str
            ID of the user to list role source groups for.

        role_id : str
            The role ID to get group sources for.

        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Group, ListOrganizationMemberRoleSourceGroupsResponseContent]
            Organization member role source groups successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = (
                await client.organizations.members.effective_roles.sources.groups.list(
                    id="id",
                    user_id="user_id",
                    from_="from",
                    take=1,
                    role_id="role_id",
                )
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            id, user_id, role_id=role_id, from_=from_, take=take, request_options=request_options
        )



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_deleted_object_0 import EventStreamCloudEventGroupDeletedObject0
from .event_stream_cloud_event_group_deleted_object_1 import EventStreamCloudEventGroupDeletedObject1
from .event_stream_cloud_event_group_deleted_object_2 import EventStreamCloudEventGroupDeletedObject2

EventStreamCloudEventGroupDeletedObject = typing.Union[
    EventStreamCloudEventGroupDeletedObject0,
    EventStreamCloudEventGroupDeletedObject1,
    EventStreamCloudEventGroupDeletedObject2,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.client_credential import ClientCredential
from ...types.client_credential_type_enum import ClientCredentialTypeEnum
from ...types.get_client_credential_response_content import GetClientCredentialResponseContent
from ...types.patch_client_credential_response_content import PatchClientCredentialResponseContent
from ...types.post_client_credential_response_content import PostClientCredentialResponseContent
from ...types.public_key_credential_algorithm_enum import PublicKeyCredentialAlgorithmEnum
from .raw_client import AsyncRawCredentialsClient, RawCredentialsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CredentialsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCredentialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCredentialsClient
        """
        return self._raw_client

    def list(
        self, client_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientCredential]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientCredential]
            Credentials successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.clients.credentials.list(
            client_id="client_id",
        )
        """
        _response = self._raw_client.list(client_id, request_options=request_options)
        return _response.data

    def create(
        self,
        client_id: str,
        *,
        credential_type: ClientCredentialTypeEnum,
        name: typing.Optional[str] = OMIT,
        subject_dn: typing.Optional[str] = OMIT,
        pem: typing.Optional[str] = OMIT,
        alg: typing.Optional[PublicKeyCredentialAlgorithmEnum] = OMIT,
        parse_expiry_from_cert: typing.Optional[bool] = OMIT,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        kid: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostClientCredentialResponseContent:
        """
        Create a client credential associated to your application. Credentials can be used to configure Private Key JWT and mTLS authentication methods, as well as for JWT-secured Authorization requests.

        **Public Key**

        Public Key credentials can be used to set up Private Key JWT client authentication and JWT-secured Authorization requests.

        Sample:

        ```json
        {
          "credential_type": "public_key",
          "name": "string",
          "pem": "string",
          "alg": "RS256",
          "parse_expiry_from_cert": false,
          "expires_at": "2022-12-31T23:59:59Z"
        }
        ```

        **Certificate (CA-signed & self-signed)**

        Certificate credentials can be used to set up mTLS client authentication. CA-signed certificates can be configured either with a signed certificate or with just the certificate Subject DN.

        CA-signed Certificate Sample (pem):

        ```json
        {
          "credential_type": "x509_cert",
          "name": "string",
          "pem": "string"
        }
        ```

        CA-signed Certificate Sample (subject_dn):

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "subject_dn": "string"
        }
        ```

        Self-signed Certificate Sample:

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "pem": "string"
        }
        ```

        The credential will be created but not yet enabled for use until you set the corresponding properties in the client:

        - To enable the credential for Private Key JWT or mTLS authentication methods, set the `client_authentication_methods` property on the client. For more information, read [Configure Private Key JWT Authentication](https://auth0.com/docs/get-started/applications/configure-private-key-jwt) and [Configure mTLS Authentication](https://auth0.com/docs/get-started/applications/configure-mtls)
        - To enable the credential for JWT-secured Authorization requests, set the `signed_request_object`property on the client. For more information, read [Configure JWT-secured Authorization Requests (JAR)](https://auth0.com/docs/get-started/applications/configure-jar)

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_type : ClientCredentialTypeEnum

        name : typing.Optional[str]
            Friendly name for a credential.

        subject_dn : typing.Optional[str]
            Subject Distinguished Name. Mutually exclusive with `pem` property. Applies to `cert_subject_dn` credential type.

        pem : typing.Optional[str]
            PEM-formatted public key (SPKI and PKCS1) or X509 certificate. Must be JSON escaped.

        alg : typing.Optional[PublicKeyCredentialAlgorithmEnum]

        parse_expiry_from_cert : typing.Optional[bool]
            Parse expiry from x509 certificate. If true, attempts to parse the expiry date from the provided PEM. Applies to `public_key` credential type.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential. If not specified (not recommended), the credential never expires. Applies to `public_key` credential type.

        kid : typing.Optional[str]
            Optional kid (Key ID), used to uniquely identify the credential. If not specified, a kid value will be auto-generated. The kid header parameter in JWTs sent by your client should match this value. Valid format is [0-9a-zA-Z-_]{10,64}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostClientCredentialResponseContent
            Credential successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.clients.credentials.create(
            client_id="client_id",
            credential_type="public_key",
        )
        """
        _response = self._raw_client.create(
            client_id,
            credential_type=credential_type,
            name=name,
            subject_dn=subject_dn,
            pem=pem,
            alg=alg,
            parse_expiry_from_cert=parse_expiry_from_cert,
            expires_at=expires_at,
            kid=kid,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetClientCredentialResponseContent:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClientCredentialResponseContent
            Credential successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.clients.credentials.get(
            client_id="client_id",
            credential_id="credential_id",
        )
        """
        _response = self._raw_client.get(client_id, credential_id, request_options=request_options)
        return _response.data

    def delete(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.clients.credentials.delete(
            client_id="client_id",
            credential_id="credential_id",
        )
        """
        _response = self._raw_client.delete(client_id, credential_id, request_options=request_options)
        return _response.data

    def update(
        self,
        client_id: str,
        credential_id: str,
        *,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchClientCredentialResponseContent:
        """
        Change a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchClientCredentialResponseContent
            Credential successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.clients.credentials.update(
            client_id="client_id",
            credential_id="credential_id",
        )
        """
        _response = self._raw_client.update(
            client_id, credential_id, expires_at=expires_at, request_options=request_options
        )
        return _response.data


class AsyncCredentialsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCredentialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCredentialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCredentialsClient
        """
        return self._raw_client

    async def list(
        self, client_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[ClientCredential]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[ClientCredential]
            Credentials successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.clients.credentials.list(
                client_id="client_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(client_id, request_options=request_options)
        return _response.data

    async def create(
        self,
        client_id: str,
        *,
        credential_type: ClientCredentialTypeEnum,
        name: typing.Optional[str] = OMIT,
        subject_dn: typing.Optional[str] = OMIT,
        pem: typing.Optional[str] = OMIT,
        alg: typing.Optional[PublicKeyCredentialAlgorithmEnum] = OMIT,
        parse_expiry_from_cert: typing.Optional[bool] = OMIT,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        kid: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostClientCredentialResponseContent:
        """
        Create a client credential associated to your application. Credentials can be used to configure Private Key JWT and mTLS authentication methods, as well as for JWT-secured Authorization requests.

        **Public Key**

        Public Key credentials can be used to set up Private Key JWT client authentication and JWT-secured Authorization requests.

        Sample:

        ```json
        {
          "credential_type": "public_key",
          "name": "string",
          "pem": "string",
          "alg": "RS256",
          "parse_expiry_from_cert": false,
          "expires_at": "2022-12-31T23:59:59Z"
        }
        ```

        **Certificate (CA-signed & self-signed)**

        Certificate credentials can be used to set up mTLS client authentication. CA-signed certificates can be configured either with a signed certificate or with just the certificate Subject DN.

        CA-signed Certificate Sample (pem):

        ```json
        {
          "credential_type": "x509_cert",
          "name": "string",
          "pem": "string"
        }
        ```

        CA-signed Certificate Sample (subject_dn):

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "subject_dn": "string"
        }
        ```

        Self-signed Certificate Sample:

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "pem": "string"
        }
        ```

        The credential will be created but not yet enabled for use until you set the corresponding properties in the client:

        - To enable the credential for Private Key JWT or mTLS authentication methods, set the `client_authentication_methods` property on the client. For more information, read [Configure Private Key JWT Authentication](https://auth0.com/docs/get-started/applications/configure-private-key-jwt) and [Configure mTLS Authentication](https://auth0.com/docs/get-started/applications/configure-mtls)
        - To enable the credential for JWT-secured Authorization requests, set the `signed_request_object`property on the client. For more information, read [Configure JWT-secured Authorization Requests (JAR)](https://auth0.com/docs/get-started/applications/configure-jar)

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_type : ClientCredentialTypeEnum

        name : typing.Optional[str]
            Friendly name for a credential.

        subject_dn : typing.Optional[str]
            Subject Distinguished Name. Mutually exclusive with `pem` property. Applies to `cert_subject_dn` credential type.

        pem : typing.Optional[str]
            PEM-formatted public key (SPKI and PKCS1) or X509 certificate. Must be JSON escaped.

        alg : typing.Optional[PublicKeyCredentialAlgorithmEnum]

        parse_expiry_from_cert : typing.Optional[bool]
            Parse expiry from x509 certificate. If true, attempts to parse the expiry date from the provided PEM. Applies to `public_key` credential type.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential. If not specified (not recommended), the credential never expires. Applies to `public_key` credential type.

        kid : typing.Optional[str]
            Optional kid (Key ID), used to uniquely identify the credential. If not specified, a kid value will be auto-generated. The kid header parameter in JWTs sent by your client should match this value. Valid format is [0-9a-zA-Z-_]{10,64}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostClientCredentialResponseContent
            Credential successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.clients.credentials.create(
                client_id="client_id",
                credential_type="public_key",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            client_id,
            credential_type=credential_type,
            name=name,
            subject_dn=subject_dn,
            pem=pem,
            alg=alg,
            parse_expiry_from_cert=parse_expiry_from_cert,
            expires_at=expires_at,
            kid=kid,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetClientCredentialResponseContent:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetClientCredentialResponseContent
            Credential successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.clients.credentials.get(
                client_id="client_id",
                credential_id="credential_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(client_id, credential_id, request_options=request_options)
        return _response.data

    async def delete(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Delete a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.clients.credentials.delete(
                client_id="client_id",
                credential_id="credential_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(client_id, credential_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        client_id: str,
        credential_id: str,
        *,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PatchClientCredentialResponseContent:
        """
        Change a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PatchClientCredentialResponseContent
            Credential successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.clients.credentials.update(
                client_id="client_id",
                credential_id="credential_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            client_id, credential_id, expires_at=expires_at, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_verification_email_response_content import CreateVerificationEmailResponseContent
from ...types.identity import Identity
from .raw_client import AsyncRawVerificationEmailClient, RawVerificationEmailClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VerificationEmailClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVerificationEmailClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVerificationEmailClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVerificationEmailClient
        """
        return self._raw_client

    def create(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVerificationEmailResponseContent:
        """
        Send an email to the specified user that asks them to click a link to [verify their email address](https://auth0.com/docs/email/custom#verification-email).

        Note: You must have the `Status` toggle enabled for the verification email template for the email to be sent.

        Parameters
        ----------
        user_id : str
            user_id of the user to send the verification email to.

        client_id : typing.Optional[str]
            client_id of the client (application). If no value provided, the global Client ID will be used.

        identity : typing.Optional[Identity]

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVerificationEmailResponseContent
            Job successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.jobs.verification_email.create(
            user_id="user_id",
        )
        """
        _response = self._raw_client.create(
            user_id=user_id,
            client_id=client_id,
            identity=identity,
            organization_id=organization_id,
            request_options=request_options,
        )
        return _response.data


class AsyncVerificationEmailClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVerificationEmailClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVerificationEmailClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVerificationEmailClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        user_id: str,
        client_id: typing.Optional[str] = OMIT,
        identity: typing.Optional[Identity] = OMIT,
        organization_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateVerificationEmailResponseContent:
        """
        Send an email to the specified user that asks them to click a link to [verify their email address](https://auth0.com/docs/email/custom#verification-email).

        Note: You must have the `Status` toggle enabled for the verification email template for the email to be sent.

        Parameters
        ----------
        user_id : str
            user_id of the user to send the verification email to.

        client_id : typing.Optional[str]
            client_id of the client (application). If no value provided, the global Client ID will be used.

        identity : typing.Optional[Identity]

        organization_id : typing.Optional[str]
            (Optional) Organization ID – the ID of the Organization. If provided, organization parameters will be made available to the email template and organization branding will be applied to the prompt. In addition, the redirect link in the prompt will include organization_id and organization_name query string parameters.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateVerificationEmailResponseContent
            Job successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.verification_email.create(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            user_id=user_id,
            client_id=client_id,
            identity=identity,
            organization_id=organization_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_member_deleted_object_member_0 import (
    EventStreamCloudEventGroupMemberDeletedObjectMember0,
)
from .event_stream_cloud_event_group_member_deleted_object_member_1 import (
    EventStreamCloudEventGroupMemberDeletedObjectMember1,
)

EventStreamCloudEventGroupMemberDeletedObjectMember = typing.Union[
    EventStreamCloudEventGroupMemberDeletedObjectMember0, EventStreamCloudEventGroupMemberDeletedObjectMember1
]



# This file was auto-generated by Fern from our API Definition.

import typing

GuardianFactorsProviderSmsProviderEnum = typing.Union[
    typing.Literal["auth0", "twilio", "phone-message-hook"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_connection_created_object_1_options_attribute_map import (
    EventStreamCloudEventConnectionCreatedObject1OptionsAttributeMap,
)
from .event_stream_cloud_event_connection_created_object_1_options_connection_settings import (
    EventStreamCloudEventConnectionCreatedObject1OptionsConnectionSettings,
)
from .event_stream_cloud_event_connection_created_object_1_options_dpop_signing_alg_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsDpopSigningAlgEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_federated_connections_access_tokens import (
    EventStreamCloudEventConnectionCreatedObject1OptionsFederatedConnectionsAccessTokens,
)
from .event_stream_cloud_event_connection_created_object_1_options_id_token_signed_response_algs_item_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsIdTokenSignedResponseAlgsItemEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_oidc_metadata import (
    EventStreamCloudEventConnectionCreatedObject1OptionsOidcMetadata,
)
from .event_stream_cloud_event_connection_created_object_1_options_schema_version_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsSchemaVersionEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_set_user_root_attributes_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsSetUserRootAttributesEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_token_endpoint_auth_method_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointAuthMethodEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_token_endpoint_auth_signing_alg_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointAuthSigningAlgEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_token_endpoint_jwtca_aud_format_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointJwtcaAudFormatEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_type_enum import (
    EventStreamCloudEventConnectionCreatedObject1OptionsTypeEnum,
)
from .event_stream_cloud_event_connection_created_object_1_options_upstream_params import (
    EventStreamCloudEventConnectionCreatedObject1OptionsUpstreamParams,
)


class EventStreamCloudEventConnectionCreatedObject1Options(UniversalBaseModel):
    """
    Options for the 'okta' connection
    """

    authorization_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's OAuth 2.0 authorization endpoint where users are redirected for authentication. Must be a valid HTTPS URL. This endpoint initiates the OAuth 2.0 authorization code flow.
    """

    client_id: str = pydantic.Field()
    """
    OAuth 2.0 client identifier issued by the identity provider during application registration. This value identifies your Auth0 connection to the identity provider.
    """

    connection_settings: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsConnectionSettings] = None
    domain_aliases: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Email domains associated with this connection for Home Realm Discovery (HRD). When a user's email matches one of these domains, they are automatically routed to this connection during authentication.
    """

    dpop_signing_alg: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsDpopSigningAlgEnum] = None
    federated_connections_access_tokens: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject1OptionsFederatedConnectionsAccessTokens
    ] = None
    icon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    https url of the icon to be shown
    """

    id_token_session_expiry_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the identity provider supports session expiry via the id_token. If true, the system will use the session_expiry claim in the id_token to determine session expiry.
    """

    id_token_signed_response_algs: typing.Optional[
        typing.List[EventStreamCloudEventConnectionCreatedObject1OptionsIdTokenSignedResponseAlgsItemEnum]
    ] = pydantic.Field(default=None)
    """
    List of algorithms allowed to verify the ID tokens. Applicable when strategy=oidc or okta.
    """

    issuer: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identity provider's unique issuer identifier URL (e.g., https://accounts.google.com). Must match the 'iss' claim in ID tokens from the identity provider.
    """

    jwks_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's JSON Web Key Set (JWKS) endpoint containing public keys for signature verification. Auth0 retrieves these keys to validate ID token signatures.
    """

    non_persistent_attrs: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
    """

    oidc_metadata: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsOidcMetadata] = None
    schema_version: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsSchemaVersionEnum] = None
    scope: typing.Optional[str] = pydantic.Field(default=None)
    """
    Space-separated list of OAuth 2.0 scopes requested during authorization. Must include 'openid' (required by OIDC spec). Common values: 'openid profile email'. Additional scopes depend on the identity provider.
    """

    send_back_channel_nonce: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true and type is 'back_channel', includes a cryptographic nonce in authorization requests to prevent replay attacks. The identity provider must include this nonce in the ID token for validation.
    """

    set_user_root_attributes: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject1OptionsSetUserRootAttributesEnum
    ] = None
    tenant_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Tenant domain
    """

    token_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's OAuth 2.0 token endpoint where authorization codes are exchanged for access tokens. Must be a valid HTTPS URL. Required for authorization code flow but optional for implicit flow.
    """

    token_endpoint_auth_method: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointAuthMethodEnum
    ] = None
    token_endpoint_auth_signing_alg: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointAuthSigningAlgEnum
    ] = None
    token_endpoint_jwtca_aud_format: typing.Optional[
        EventStreamCloudEventConnectionCreatedObject1OptionsTokenEndpointJwtcaAudFormatEnum
    ] = None
    upstream_params: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsUpstreamParams] = None
    userinfo_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional URL of the identity provider's UserInfo endpoint. When configured with attribute mapping, Auth0 calls this endpoint to retrieve additional user profile claims using the access token.
    """

    attribute_map: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsAttributeMap] = None
    domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    Domain of the Okta organization (e.g., dev-123456.okta.com). Should be just the domain of the okta server with no scheme or trailing backslash. Discovery runs only when connection.options.oidc_metadata is empty and a domain is provided
    """

    type: typing.Optional[EventStreamCloudEventConnectionCreatedObject1OptionsTypeEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionSigningCertificatePemPingFederate = str
"""
Base64-encoded X.509 certificate from the identity provider used to validate signatures in SAML responses and assertions. The certificate is decoded and used for cryptographic signature verification.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .domain_certificate_authority_enum import DomainCertificateAuthorityEnum
from .domain_certificate_status_enum import DomainCertificateStatusEnum


class DomainCertificate(UniversalBaseModel):
    """
    Certificate information. This object is relevant only for Custom Domains with Auth0-Managed Certificates.
    """

    status: typing.Optional[DomainCertificateStatusEnum] = None
    error_msg: typing.Optional[str] = pydantic.Field(default=None)
    """
    A user-friendly error message will be presented if the certificate status is provisioning_failed or renewing_failed.
    """

    certificate_authority: typing.Optional[DomainCertificateAuthorityEnum] = None
    renews_before: typing.Optional[str] = pydantic.Field(default=None)
    """
    The certificate will be renewed prior to this date.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

SetSelfServiceProfileCustomTextRequestContent = typing.Dict[str, str]
"""
The list of text keys and values to customize the Self-Service Enterprise Configuration flow page. Values can be plain text or rich HTML content limited to basic styling tags and hyperlinks.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

FlowActionSlackPostMessageParamsAttachmentColor = typing.Union[typing.Literal["GOOD", "WARNING", "DANGER"], typing.Any]



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_salesforce_create_lead import FlowActionSalesforceCreateLead
from .flow_action_salesforce_get_lead import FlowActionSalesforceGetLead
from .flow_action_salesforce_search_leads import FlowActionSalesforceSearchLeads
from .flow_action_salesforce_update_lead import FlowActionSalesforceUpdateLead

FlowActionSalesforce = typing.Union[
    FlowActionSalesforceCreateLead,
    FlowActionSalesforceGetLead,
    FlowActionSalesforceSearchLeads,
    FlowActionSalesforceUpdateLead,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .log_stream_pii_algorithm_enum import LogStreamPiiAlgorithmEnum
from .log_stream_pii_log_fields_enum import LogStreamPiiLogFieldsEnum
from .log_stream_pii_method_enum import LogStreamPiiMethodEnum


class LogStreamPiiConfig(UniversalBaseModel):
    log_fields: typing.List[LogStreamPiiLogFieldsEnum]
    method: typing.Optional[LogStreamPiiMethodEnum] = None
    algorithm: typing.Optional[LogStreamPiiAlgorithmEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
